

var Template

Template = class Template {
  static updateContent(data) {
    var count, key, ref, value
    count = 0
    ref = Template.decodeData(data)
    for (key in ref) {
      value = ref[key]
      count += Template.updateContentForKey(key, value)
    }
    return count
  }

  static decodeData(data) {
    return JSON.parse(decodeURIComponent(data))
  }

  static updateContentForKey(key, content) {
    switch (key) {
      case "date":
        return Template.updateFormattedContentForKey(key, content)
      default:
        return Template.updatePlainContentForKey(key, content)
    }
  }

  static updatePlainContentForKey(key, content) {
    var element, elements, i, len
    elements = Template.getElementsForKey(key)
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      element.innerHTML = content
      Template.dispatchChangeEvent(element)
    }
    return elements.length
  }

  static updateFormattedContentForKey(key, content) {
    var element, elements, format, i, len
    elements = Template.getElementsForKey(key)
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      format = element.dataset[key]
      if ((format != null ? format.length : void 0) === 0) {
        format = "default"
      }
      element.innerHTML = content[format]
      Template.dispatchChangeEvent(element)
    }
    return elements.length
  }

  static updateInset(height) {
    document.documentElement.style.paddingTop = height
    return document.documentElement.style.paddingBottom = height
  }

  static dispatchChangeEventForKey(key) {
    var element, elements, i, len
    elements = Template.getElementsForKey(key)
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      Template.dispatchChangeEvent(element)
    }
    return elements.length
  }

  static dispatchChangeEvent(element) {
    var changeEvent
    changeEvent = new Event("ia-writer-change")
    return element.dispatchEvent(changeEvent)
  }

  static getElementsForKey(key) {
    return document.querySelectorAll(`[data-${key}]`)
  }

  static getFormatsForKey(key) {
    var element, elements, format, formats, i, len
    elements = Template.getElementsForKey(key)
    formats = new Set()
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      format = element.dataset[key]
      if (format.length > 0) {
        formats.add(format)
      } else {
        formats.add("default")
      }
    }
    if (formats.size == 0) {
      return "";
    }
    return JSON.stringify(Array.from(formats))
  }
  
  // Inserts stylesheets into `<head>`.
  // - parameters:
  //   - data: URL-encoded JSON string with an array of objects, each with a `title` and `style` properties
  static insertStylesheets(data) {
    var stylesheetArray = Template.decodeData(data)
    for (var stylesheet of stylesheetArray) {
      var style = document.createElement("style")
      style.dataset.id = stylesheet.id
      style.innerHTML = stylesheet.style
      document.head.appendChild(style)
    }
  }

  // Expands contents of `.math` expressions using KaTeX.
  // - parameters:
  //   - format: null renders default output, `mathml` removes HTML math tree and leaves MathML
  static updateMath(format) {
    var delimiters = [
      {left: "$$", right: "$$", display: true},
      {left: "\\(", right: "\\)", display: false},
      {left: "$", right: "$", display: false},
      {left: "\\[", right: "\\]", display: true}
    ]
    var element, elements, i, len, macros
    elements = document.body.getElementsByClassName("math")
    macros = {}
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      var mathExpression = element.textContent
      var math = ""
      var display = false
      for (var delimiter of delimiters) {
        if (mathExpression.startsWith(delimiter.left) && mathExpression.endsWith(delimiter.right)) {
          math = mathExpression.substr(delimiter.left.length, mathExpression.length - delimiter.left.length - delimiter.right.length)
          display = delimiter.display
          break
        }
      }
      if (math.length > 0) {
        katex.render(math, element, {
          displayMode: display,
          throwOnError: false,
          trust: true,
          macros: macros
        })
        if (format == "mathml") {
            var mathElement = element.getElementsByClassName("katex-mathml")[0].getElementsByTagName("math")[0]
            while (element.hasChildNodes()) {
                element.removeChild(element.lastChild)
            }
            element.appendChild(mathElement)
        }
        element.lang = "und-Zmth"
        element.dataset.displayMath = display
        Template.dispatchChangeEvent(element)
      }
    }
  }

  static updateSmartTables() {
    var element, elements, i, len
    elements = document.body.getElementsByTagName("table")
    for (i = 0, len = elements.length; i < len; i++) {
      element = elements[i]
      SmartTables.processTable(element, [i])
      Template.dispatchChangeEvent(element)
    }
  }
  
  static updateClass(data) {
    var classArray = Template.decodeData(data)
    document.documentElement.className = classArray.join(" ")
  }

  static updateLang(data) {
    var langArray = Template.decodeData(data)
    document.documentElement.lang = langArray.join(", ")
  }

  static updateOptions(data) {
    var classList, className, ref, value
    classList = document.documentElement.classList
    ref = Template.decodeData(data)
    for (className in ref) {
      value = ref[className]
      classList.toggle(className, value > 0)
    }
  }
}
